@@ -3,14 +3,17 @@ use strict;
 use warnings;
 use utf8;
 use 5.008;
-our $VERSION = '0.62';
+our $VERSION = '0.63';
 
 use Carp;
 use JSON;
 use URI;
 use URI::Escape;
 use Digest::SHA;
+use Time::Piece;
 use AnyEvent::HTTP;
+use HTTP::Request::Common 'POST';
+use Data::Recursive::Encode;
 
 use Net::OAuth;
 use Net::OAuth::ProtectedResourceRequest;
@@ -27,9 +30,15 @@ our %PATH = (
     authenticate  => 'https://api.twitter.com/oauth/authenticate',
 );
 
+our %RESOURCE_URL_BASE = (
+    '1.0' => 'http://api.twitter.com/1/%s.json',
+    '1.1' => 'https://api.twitter.com/1.1/%s.json',
+);
+
 sub new {
     my ($class, %args) = @_;
 
+    $args{api_version}         ||= '1.1';
     $args{access_token}        ||= $args{token};
     $args{access_token_secret} ||= $args{token_secret};
 
@@ -67,47 +76,73 @@ sub request {
     ($opt{api} || $opt{url})
         or Carp::croak "'api' or 'url' option is required";
 
-    my $url = $opt{url} || 'http://api.twitter.com/1/' . $opt{api} . '.json';
+    my $url_base = $RESOURCE_URL_BASE{ $self->{api_version} };
+    my $url = $opt{url} || sprintf $url_base, $opt{api};
 
     ref $cb eq 'CODE'
         or Carp::croak "callback coderef is required";
 
-    $opt{params} ||= {};
-    ref $opt{params} eq 'HASH'
-        or Carp::croak "parameters must be hashref.";
+    my $params = $opt{params} || {};
+    my $is_multipart = ref $params eq 'ARRAY';
 
-    $opt{method} = uc $opt{method};
-    $opt{method} =~ /^(?:GET|POST)$/
+    my $method = uc $opt{method};
+    $method =~ /^(?:GET|POST)$/
         or Carp::croak "'method' option should be GET or POST";
 
     my $req = $self->_make_oauth_request(
         class => 'Net::OAuth::ProtectedResourceRequest',
         request_url     => $url,
-        request_method  => $opt{method},
-        extra_params    => $opt{params},
+        request_method  => $method,
+        extra_params    => ($is_multipart ? {} : $params),
         consumer_key    => $self->{consumer_key},
         consumer_secret => $self->{consumer_secret},
         token           => $self->{access_token},
         token_secret    => $self->{access_token_secret},
     );
 
-    my %req_params;
-    if ($opt{method} eq 'POST') {
+    my $req_params = {};
+
+    if ($method eq 'POST') {
         $url = $req->normalized_request_url;
-        $req_params{body} = $req->to_post_body;
+
+        if ($is_multipart) {
+            my $encoded_params = Data::Recursive::Encode::_apply(
+                sub { utf8::is_utf8($_[0]) ? Encode::encode_utf8($_[0]) : $_[0] },
+                {},
+                $params
+            );
+
+            my $ireq = POST(
+                $url,
+                Content_Type => 'multipart/form-data',
+                Content => [ @$encoded_params ]
+            );
+
+            $req_params->{body} = $ireq->content;
+            $req_params->{headers} = {
+                Authorization  => $req->to_authorization_header,
+                'Content-Type' => join "; ", $ireq->content_type,
+            };
+
+        } else {
+            $req_params->{body} = $req->to_post_body;
+            $req_params->{headers}{'Content-Type'} = 'application/x-www-form-urlencoded';
+        }
+
     } else {
         $url = $req->to_url;
     }
 
-    AnyEvent::HTTP::http_request $opt{method} => $url, %req_params, sub {
+    AnyEvent::HTTP::http_request $method => $url, %$req_params, sub {
         my ($body, $hdr) = @_;
 
+        local $@;
+        my $json = eval { JSON::decode_json($body) };
+
         if ($hdr->{Status} =~ /^2/) {
-            local $@;
-            my $json = eval { JSON::decode_json($body) };
             $cb->($hdr, $json, $@ ? "parse error: $@" : $hdr->{Reason});
         } else {
-            $cb->($hdr, undef, $hdr->{Reason});
+            $cb->($hdr, undef, $hdr->{Reason}, $json);
         }
     };
 
@@ -209,6 +244,11 @@ sub _parse_response {
     return %query;
 }
 
+sub parse_timestamp { # Twitter uses weird created_at format: "Thu Mar 01 17:38:56 +0000 2012"
+    my ($class, $created_at) = @_;
+    localtime( Time::Piece->strptime($created_at, '%a %b %d %H:%M:%S %z %Y' )->epoch )
+}
+
 1;
 __END__
 
@@ -327,11 +367,17 @@ AnyEvent::Twitter - A thin wrapper for Twitter API using OAuth
 
 AnyEvent::Twitter is a very thin wrapper for Twitter API using OAuth.
 
+=head1 API VERSION
+
+As of version 0.63, L<AnyEvent::Twitter> supports Twitter REST API v1.1.
+
+NOTE: API version 1.0 is already deprecated.
+
 =head1 METHODS
 
 =head2 new
 
-All arguments are required.
+All arguments are required except C<api_version>.
 If you don't know how to obtain these parameters, take a look at eg/gen_token.pl and run it.
 
 =over 4
@@ -344,6 +390,11 @@ If you don't know how to obtain these parameters, take a look at eg/gen_token.pl
 
 =item C<access_token_secret> (or C<token_secret>)
 
+=item C<api_version> (optional; default: 1.1)
+
+If you have a problem with API changes, specify C<api_version> parameter.
+Possible values are: C<1.1> or C<1.0>
+
 =back
 
 =head2 get
@@ -368,8 +419,32 @@ If you don't know how to obtain these parameters, take a look at eg/gen_token.pl
 
 =item C<< $ua->post($url, \%params, sub {}) >>
 
+=item C<< $ua->post($api, \@params, sub {}) >>
+
+=item C<< $ua->post($url, \@params, sub {}) >>
+
 =back
 
+=head3 UPLOADING MEDIA FILE
+
+You can use C<statuses/update_with_media> API to upload photos by specifying parameters as arrayref like below example.
+
+Uploading photos will be tranferred with Content-Type C<multipart/form-data> (not C<application/x-www-form-urlencoded>)
+
+    use utf8;
+    $ua->post(
+        'statuses/update_with_media',
+        [
+            status    => '桜',
+            'media[]' => [ undef, $filename, Content => $loaded_image_binary ],
+        ],
+        sub {
+            my ($hdr, $res, $reason) = @_;
+            say $res->{user}{screen_name};
+        }
+    );
+
+
 =head2 request
 
 These parameters are required.
@@ -384,9 +459,10 @@ If you want to specify the API C<url>, the C<url> parameter is good for you. The
 
 The C<api> parameter will be internally processed as:
 
-    $url = 'http://api.twitter.com/1/' . $opt{api} . '.json';
+    sprintf 'https://api.twitter.com/1.1/%s.json', $api; # version 1.1
+    sprintf 'http://api.twitter.com/1/%s.json',    $api; # version 1.0
 
-You can check available C<api>s at L<API Documentation|https://dev.twitter.com/docs/api>
+You can find available C<api>s at L<API Documentation|https://dev.twitter.com/docs/api>
 
 =item C<method> and C<params>
 
@@ -395,20 +471,36 @@ Then specify it. GET and POST methods are allowed. You can omit C<params> if Twi
 
 =item callback
 
-This module is AnyEvent::HTTP style, so you have to pass the callback (coderef).
+This module is L<AnyEvent::HTTP> style, so you have to pass the callback (coderef).
 
-Passed callback will be called with C<$header>, C<$response> and C<$reason>.
-If something is wrong with the response from Twitter API, C<$response> will be C<undef>. So you can check the value like below.
+Passed callback will be called with C<$header>, C<$response>, C<$reason> and C<$error_response>.
+If something is wrong with the response from Twitter API, C<$response> will be C<undef>.
+On non-2xx HTTP status code, you can get the decoded response via C<$error_response>.
+So you can check the value like below.
 
-    sub {
-        my ($header, $response, $reason) = @_;
+    my $callback = sub {
+        my ($header, $response, $reason, $error_response) = @_;
 
         if ($response) {
             say $response->{screen_name};
         } else {
             say $reason;
+            for my $error (@{$error_response->{errors}}) {
+                say "$error->{code}: $error->{message}";
+            }
         }
-    }
+    };
+
+=back
+
+=head2 parse_timestamp
+
+C<parse_timestamp> parses C<created_at> timestamp like "Thu Mar 01 17:38:56 +0000 2012".
+It returns L<Time::Piece> object. Its timezone is localtime.
+
+=over 4
+
+=item C<< AnyEvent::Twitter->parse_timestamp($created_at) >>
 
 =back
 
